from django.conf.urls import include, patterns, url
from django.views.generic import TemplateView

from . import views


api_patterns = patterns(
    '',
    url(r'^channels$', views.ChannelListAPI.as_view(), name='api_channels'),
    url(r'^channels/(?P<slug>[a-zA-Z\d\-\_]+)$',
        views.ChannelDetailAPI.as_view(), name='api_channel_details'),
    url(r'^channels/(?P<slug>[a-zA-Z\d\-\_]+)/items$',
        views.ItemListAPI.as_view(), name='api_channel_items'),
    url(r'^items/(?P<slug>[a-zA-Z\d\-\_]+)$', views.ItemDetailAPI.as_view(),
        name='api_item_details'),
    url(r'^items/(?P<slug>[a-zA-Z\d\-\_]+)/file$',
        views.ItemFileAPI.as_view(), name='api_item_file'),
)

urlpatterns = patterns(
    '',
    url(r'^api/', include(api_patterns)),
    url(r'^', TemplateView.as_view(template_name='podcast_client/home.html'),
        name='home'),
)



from rest_framework import pagination, serializers

from podcast_client.models import PodcastChannel, PodcastItem


class DetailPaginationSerializer(pagination.PaginationSerializer):
    per_page = serializers.Field(source='paginator.per_page')
    page_count = serializers.Field(source='paginator.num_pages')


class PodcastChannelDetailSerializer(serializers.HyperlinkedModelSerializer):
    api_url = serializers.HyperlinkedIdentityField(
        view_name='podcast_client:api_channel_details', lookup_field='slug')
    items_url = serializers.HyperlinkedIdentityField(
        view_name='podcast_client:api_channel_items', lookup_field='slug')
    has_unlistened = serializers.Field(source='has_unlistened')
    latest_publish_date = serializers.SerializerMethodField(
        'get_latest_publish_date')

    class Meta:
        model = PodcastChannel
        fields = ('url', 'api_url', 'items_url', 'title', 'slug',
                  'description', 'website', 'copyright', 'cover_url',
                  'download_new', 'has_unlistened')
        read_only_fields = ('title', 'slug', 'description', 'website',
                            'copyright', 'cover_url')

    def get_latest_publish_date(self, obj):
        return obj.podcast_items.latest().publish_date


class PodcastItemDetailSerializer(serializers.HyperlinkedModelSerializer):
    media_type = serializers.Field(source='media_type')
    api_url = serializers.HyperlinkedIdentityField(
        view_name='podcast_client:api_item_details', lookup_field='slug')
    channel_url = serializers.HyperlinkedRelatedField(
        source='channel', view_name='podcast_client:api_channel_details',
        lookup_field='slug', read_only=True)
    file_downloaded = serializers.SerializerMethodField(
        'is_file_downloaded')

    class Meta:
        model = PodcastItem
        fields = (
            'url', 'api_url', 'channel_url', 'title', 'slug', 'description',
            'author', 'link', 'publish_date', 'media_type', 'listened',
            'cover_url', 'file_downloaded')
        read_only_fields = (
            'url', 'title', 'slug', 'description', 'author', 'link',
            'publish_date', 'cover_url')

    def is_file_downloaded(self, obj):
        return bool(obj and obj.file)



from django.contrib import admin

from podcast_client.models import PodcastChannel, PodcastItem


class PodcastChannelAdmin(admin.ModelAdmin):
    actions = ('update_channels',)
    list_display = ('title', 'download_new',)
    readonly_fields = ('title', 'description', 'slug', 'cover_url', 'created',
                       'modified')
    search_fields = ('title',)

    def update_channels(self, request, queryset):
        for channel in queryset:
            channel.update_channel()

    def save_model(self, request, obj, form, change):
        obj.save()
        obj.update_channel()


class PodcastItemAdmin(admin.ModelAdmin):
    actions = ('download_files', 'delete_files', 'mark_as_listened')
    list_editable = ('listened',)
    list_display = ('title', 'channel', 'publish_date', 'file', 'listened')
    list_filter = ('channel', 'listened')
    ordering = ('-publish_date',)
    readonly_fields = ('title', 'channel', 'url', 'file', 'file_type',
                       'description', 'publish_date', 'guid')
    search_fields = ('title', 'description')

    def download_files(self, request, queryset):
        for item in queryset:
            item.download_file()

    def delete_files(self, request, queryset):
        for item in queryset:
            item.delete_file()

    def mark_as_listened(self, request, queryset):
        queryset.update(listened=True)


admin.site.register(PodcastChannel, PodcastChannelAdmin)
admin.site.register(PodcastItem, PodcastItemAdmin)



from .celery import app
from .models import PodcastChannel, PodcastItem


@app.task
def update_channel(slug, force_download=False):
    channel = PodcastChannel.objects.get(slug=slug)
    channel.update_channel(download=force_download)


@app.task
def update_all_channels(force_download=False):
    for channel in PodcastChannel.objects.all():
        channel.update_channel(download=force_download)


@app.task
def download_file(item_id):
    item = PodcastItem.objects.get(id=item_id)
    item.download_file()
